*** Program: Calibration statistics and figures for risk prediction models

*** This program calculate calibration statistics of multiple models by three method, Hosmer-Lemeshow, Spiegelhalter, and Brier
*** And also draw a figure of pooled observed risk vs predicted risk of all modes

*** Last update: March 10, 2014 by Yingying Sang (Johns Hopkins Bloomberg School of Public Health, ysang1@jhu.edu)
******************************************************************************************************************

program define calmultimod, rclass
syntax varlist [if] [in], outcome(varname) [grp(integer 10) gcut(numlist) nlgh(integer 20) fu(varname) y(integer 10) name1(string) name2(string) name3(string) name4(string) name5(string) name6(string) ///
   name7(string) name8(string) name9(string) name10(string) name11(string) name12(string) name13(string) name14(string) name15(string) name16(string) name17(string) ///
   name18(string) name19(string) name20(string) lc1(string) lc2(string) lc3(string) lc4(string) lc5(string) lc6(string) lc7(string) lc8(string) lc9(string) lc10(string) ///
   lc11(string) lc12(string) lc13(string) lc14(string) lc15(string) lc16(string) lc17(string) lc18(string) lc19(string) lc20(string) pos(integer 5) nostat nograph gname(string) *]
preserve
local nv=wordcount("`varlist'")
qui sum `outcome' `if' `in'
if r(sum)<10 {
   return clear
   exit
   }
tempvar id od
if "`fu'"=="" qui gen `od'=`outcome'
else qui gen `od'=(`outcome' & `fu'<=`y')
count if `od'==1
return scalar od=r(N)
tokenize `varlist'
forvalues i=1/`nv' {
   local risk`i' ``i''
   }
forvalues i=1/`=max(20,`nv')' {
   tempvar dec`i' s`i' od`i' exd`i' num`i' H`i'
   qui gen `exd`i''=.
   qui gen `od`i''=.
   qui gen `num`i''=.
   if `i'>`nv' continue
   qui sum `risk`i'' `if' `in'
   if r(N)==0 continue
   if "`gcut'"=="" xtile `dec`i''=`risk`i'' `if' `in', nq(`grp')
   else {
      egen `dec`i''=cut(`risk`i'') `if' `in', at(0 `gcut' 1.01) label
	  replace `dec`i''=`dec`i''+1
	  local grp=wordcount("`gcut'")+1
	  }
   sts generate `s`i''=s,by(`dec`i'')
   forvalues j=1/`grp' {
	  qui sum `risk`i'' if `dec`i''==`j'
	  return scalar exd`i'`j'=r(mean)
	  qui replace `exd`i''=r(mean) in `j'
      qui sum `s`i'' if _t<=`y' & `dec`i''==`j'
	  if r(N)==0 return scalar od`i'`j'=0
	  else return scalar od`i'`j'=1-r(min)
      qui replace `od`i''=return(od`i'`j') in `j'
	  qui count if `dec`i''==`j'
	  return scalar num`i'`j'=r(N)
	  qui replace `num`i''=r(N) in `j'
	  }
   if "`stat'"=="" {
	  qui gen `H`i''=(`od`i''-`exd`i'')^2*`num`i''/`exd`i''/(1-`exd`i'')
      qui sum `H`i''
      return scalar HL_`i'=r(sum)
	  drop `H`i''
	  qui gen `H`i''=(`od'-`risk`i'')*(1-2*`risk`i'') `if' `in'
      qui sum `H`i''
	  return scalar spi_`i'=r(sum)
	  qui replace `H`i''=(1-`risk`i'')*(1-2*`risk`i'')^2*`risk`i'' `if' `in'
      qui sum `H`i''
	  return scalar spi_`i'=return(spi_`i')/sqrt(r(sum))
	  drop `H`i''
	  qui gen `H`i''=(`od'-`risk`i'')^2 `if' `in'
	  qui sum `H`i''
	  return scalar bri_`i'=r(mean)
	  if `i'>1 {
	     forvalues k=1/`=`i'-1' {
	        drop `H`i''
		    qui gen `H`i''=(`risk`i''-`risk`k'')*(`risk`i''+`risk`k''-2*`od') `if' `in'
			qui sum `H`i''
			return scalar red_`i'`k'=r(sum)
			qui replace `H`i''=(`risk`i''-`risk`k'')^2*(`risk`i''+`risk`k'')*(2-`risk`i''-`risk`k'') `if' `in'
			qui sum `H`i''
			return scalar red_`i'`k'=return(red_`i'`k')/sqrt(r(sum))
			}
		 }
	  }
   }
if "`graph'"!="" exit
if `nlgh'<`nv' local nv=`nlgh'
local ymin=1
local ymax=0
forvalues i=1/`nv' {
   local legendorder `"`legendorder'`i' "`name`i''" "'
   local ymin=min(`ymin',`od`i''[1],`exd`i''[1])
   local ymax=max(`ymax',`od`i''[`grp'],`exd`i''[`grp'])
   }
if `nv'<20 {
   forvalues i=`=`nv'+1'/20 {
      qui replace `od`i''=.
	  qui replace `exd`i''=.
	  }
   }
qui gen `id'=`ymin' in 1
qui replace `id'=`ymax' in 2
local yv=(`ymax'-`ymin')/3
if `yv'<0.05 local yv=round(`yv',0.01)
else if `yv'<0.1 local yv=round(`yv',0.02)
else if `yv'<0.2 local yv=round(`yv',0.05)
else local yv=round(`yv',0.1)
local ymin=round(`ymin',`yv')
local ymax=round(`ymax',`yv')
qui replace `id'=`ymin' in 1 if `id'[1]>`ymin'
qui replace `id'=`ymax' in 2 if `id'[2]<`ymax'
local yl "`ymin'(`yv')`ymax'"
twoway scatter `od1' `exd1', ms(o) msize(2) mc(`lc1') c(l) lc(`lc1') graphr(fc(white)) ///
	   legend(order(`legendorder') cols(1) region(color(none)) pos(`pos') ring(0) si(3)) ///
	   ylabel(`yl',labs(4) angle(0)) xlabel(`yl',labs(4) grid) `xys' ysize(5) xsize(5) ///
	   ytitle("{bf:Observed risk},`y' year",si(5) m(r+1)) xtitle("{bf:Predicted risk},`y' year",si(5) m(t+1)) `options' || ///
	   scatter `od2' `exd2', ms(o) msize(2) mc(`lc2') c(l) lc(`lc2') || ///
	   scatter `od3' `exd3', ms(o) msize(2) mc(`lc3') c(l) lc(`lc3') || ///
	   scatter `od4' `exd4', ms(o) msize(2) mc(`lc4') c(l) lc(`lc4') || ///
	   scatter `od5' `exd5', ms(o) msize(2) mc(`lc5') c(l) lc(`lc5') || ///
	   scatter `od6' `exd6', ms(o) msize(2) mc(`lc6') c(l) lc(`lc6') || ///
	   scatter `od7' `exd7', ms(o) msize(2) mc(`lc7') c(l) lc(`lc7') || ///
	   scatter `od8' `exd8', ms(o) msize(2) mc(`lc8') c(l) lc(`lc8') || ///
	   scatter `od9' `exd9', ms(o) msize(2) mc(`lc9') c(l) lc(`lc9') || ///
	   scatter `od10' `exd10', ms(o) msize(2) mc(`lc10') c(l) lc(`lc10') || ///
	   scatter `od11' `exd11', ms(o) msize(2) mc(`lc11') c(l) lc(`lc11') || ///
	   scatter `od12' `exd12', ms(o) msize(2) mc(`lc12') c(l) lc(`lc12') || ///
	   scatter `od13' `exd13', ms(o) msize(2) mc(`lc13') c(l) lc(`lc13') || ///
	   scatter `od14' `exd14', ms(o) msize(2) mc(`lc14') c(l) lc(`lc14') || ///
	   scatter `od15' `exd15', ms(o) msize(2) mc(`lc15') c(l) lc(`lc15') || ///
	   scatter `od16' `exd16', ms(o) msize(2) mc(`lc16') c(l) lc(`lc16') || ///
	   scatter `od17' `exd17', ms(o) msize(2) mc(`lc17') c(l) lc(`lc17') || ///
	   scatter `od18' `exd18', ms(o) msize(2) mc(`lc18') c(l) lc(`lc18') || ///
	   scatter `od19' `exd19', ms(o) msize(2) mc(`lc19') c(l) lc(`lc19') || ///
	   scatter `od20' `exd20', ms(o) msize(2) mc(`lc20') c(l) lc(`lc20') || ///
	   line `id' `id', lc(gray) lw(0.5)
if "`gname'"!="" graph save "`gname'.gph",replace
end
